/* Percolation Lattice class for Java version of Program Site in Gould and Tobochnik  */

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import simphy.*;

public class Lattice extends Frame implements MouseListener{
    float rsite[][];
    int L = 50, dL = 4;   // dL = size of each cell
    int cornerx = 100, cornery = 20;
    double p = 0.5;
    Image offscreen;
    Graphics buffer;
    TextArea ta;


    public Lattice() {
      setTitle("Percolation");
      setSize(500,500);
      setLayout(new BorderLayout());
	  ta = new TextArea("Cluster sizes" + "\n");
	  add("South",ta);   
	  addMouseListener(this); 
      setVisible(true);
	  offscreen = createImage(getSize().width, getSize().height);
	  buffer = offscreen.getGraphics();
    }

	public void paint(Graphics g) {
		setBackground(Color.white);
		g.drawImage(offscreen,0,0,this);
	}

    public void mouseClicked(MouseEvent e) {
       fillCluster(e.getX(),e.getY());
        }
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}

    public void newLattice() {
       if ( L <= 80)
            dL = 4;             // set size of cells on screen
       else if (L <= 110)
            dL = 3;
       else 
            dL = 2;
       buffer.clearRect (0, 0,size().width,size().height);
       buffer.drawRect(cornerx,cornery,dL*L,dL*L);
       rsite = new float[L][L];
       for(int i = 0; i < L; i++)
        for(int j = 0; j < L; j++)
            rsite[i][j] = (float) Math.random(); } // assign random values to sites

    public void fillCluster( int x, int y) {  
        x = (x-cornerx)/dL;  // convert screen coordinates to lattice coordinates
        y = (y-cornery)/dL;
        if (rsite[x][y] < p) {
            Color color = new Color((int) (Math.random()*255),(int) (Math.random()*255),
                     (int) (Math.random()*255));
            buffer.setColor(color);
            Vector cluster = new Vector();
            int[] site = {x,y};
            rsite[x][y] = -rsite[x][y];
            cluster.insertElementAt(site,0);
            formcluster(cluster); 
            ta.appendText("Size = " + String.valueOf(cluster.size()) + "\n");         
            for(int i = 0; i < cluster.size();i++) {   // draw cluster
              site = (int[]) cluster.elementAt(i);
              x = site[0];
              y = site[1];
              rsite[x][y] = -rsite[x][y];
              buffer.fillRect(cornerx+x*dL,cornery+y*dL,dL,dL);
          } }
          repaint();}
 
 public void formcluster(Vector cluster) {  // add occupied neighbors to cluster
   // non-recursive version
     int[] site;
     int nCheck = 0;   // location in cluster of site whose neighbors are being checked
     int nLast = 0;    // location in cluster of last site in cluster
       while(nCheck <= nLast) {
       site = (int[]) cluster.elementAt(nCheck);
       int x = site[0];
       int y = site[1];
        nLast = addTo(cluster,x+1,y,nCheck,nLast);
        nLast = addTo(cluster,x-1,y,nCheck,nLast);
        nLast = addTo(cluster,x,y+1,nCheck,nLast);
        nLast = addTo(cluster,x,y-1,nCheck,nLast);
        nCheck++; } }
 
 public int addTo(Vector cluster, int x, int y, int nCheck, int nLast) { 
     if ((x >= 0) && (x < L) && (y >= 0) && (y < L ))
       if (rsite[x][y] < p && rsite[x][y] > 0) {   // add site to cluster
              int[] site = {x,y};
              rsite[x][y] = -rsite[x][y];
              cluster.insertElementAt(site,nCheck+1);
              return nLast+1;}
     return nLast;} // neighbor not in cluster 
              
public void drawSites() {          // draw occupied sites
     buffer.setColor(Color.black);
     for(int i = 0; i < L; i++)
       for(int j = 0; j < L; j++) {
          rsite[i][j] = Math.abs(rsite[i][j]);
          if (rsite[i][j] < p) 
             buffer.fillRect(cornerx+i*dL,cornery+j*dL,dL,dL); 
             }
     buffer.drawRect(cornerx,cornery,dL*L,dL*L);
     buffer.drawString("Click on an occupied cell to see cluster",100,50+L*dL);
     repaint();
     }
     
}
